(*****************************************************************************)
(*                       bande.ml                                           *)
(*****************************************************************************)

(*inclusion of bibliotheque*)

open Input_util
open Format
open Algebra
open Vector
open Matrix

(************************************************************************)
(*                   Module Type Bande_Matrix                           *)
(************************************************************************)

module Make(R:Ring) = 
  functor (Domain : Vector with type scalar = R.elem) ->
  functor (Image : Vector with type scalar = R.elem) ->
struct
  type scalar = R.elem
  type argument = Domain.elem
  type result = Image.elem
		     
  let columns = Domain.dim
  let lines = Image.dim
  module D = struct 
    let dim = columns * lines 
  end
  let dim = columns * lines

  type elem = R.elem array array
		
  let mini = min columns lines

  let maxi = max columns lines

  let line_length k = 
    if k < mini then k + 1
    else if k >= maxi then columns + lines - k - 1
    else mini

  let coor_bande i j = 
    let k = j - i + columns - 1 in
    let l = if k < columns then j else i in
    k, l
 
  let coor_matrix k l =
    if k < columns then
      (l + columns - 1 - k, l)
    else
      (l, l - columns + 1 + k)

  let create init =
     let m = Array.create (lines + columns - 1) [||] in
     let g i j v =
       let k, l = coor_bande i j in
       if Array.length m.(k)<1 then
	   m.(k)<-Array.create (line_length k) R.zero;
       m.(k).(l)<-v; 
     in 
     init g;
     m
	 	          
   let null () =
     let m = Array.create (lines + columns - 1) [||] in
     for j= 0 to lines + columns - 2 do
       m.(j)<- Array.create 0 R.zero 
     done;
     m
	          
   let copy m =      
     let r = Array.create (lines + columns - 1) [||] in
     for j = 0 to lines + columns - 2 do
       r.(j)<- Array.copy m.(j)
     done;
     r
	  
   let apply m v =
     let result = Image.null () in
     for k=0 to lines + columns - 2 do
       for l=0 to Array.length m.(k) - 1 do
	 let i, j = coor_matrix k l in
	 Image.vput result j 
	   (R.(++) 
	      (Image.vget result j) 
	      (R.( ** ) m.(k).(l) (Domain.vget v i)))
       done
     done;
     result

   let tapply v m =
     let result = Domain.null () in
     for k=0 to lines + columns - 2 do
       for l=0 to Array.length m.(k) - 1 do
	 let i, j = coor_matrix k l in
	 Domain.vput result i 
	   (R.(++) 
	      (Domain.vget result i) 
	      (R.( ** ) (R.conjugate m.(k).(l)) (Image.vget v j)))
       done
     done;
     result

   let mul_scal x m =
     for k = 0 to lines + columns - 2 do
       for l = 0 to ((Array.length m.(k))-1)do
	 m.(k).(l)<- R.( ** ) x m.(k).(l)
       done
     done
	 
   let write formatter m =
     let next = Array.create (lines + columns - 1) 0 in
     let k' = ref (lines + columns - 2)  in
     for k = lines + columns - 2 downto 0 do
       if Array.length m.(k)>=1 then 
	 for k'' = !k' downto k do 
	   next.(k'') <- k;
	   k' := k - 1
	 done
     done; 
     for k'' = !k' downto 0 do 
       next.(k'') <- (-1)
     done;
     pp_print_string formatter "[";
     pp_open_hvbox formatter 2;
     pp_print_int formatter columns;
     pp_print_string formatter "x";
     pp_print_int formatter lines;
     pp_print_string formatter ":";
     for j=0 to lines-1 do
       if j <> 0 then pp_print_string formatter ",";
       pp_print_space formatter ();
       pp_print_string formatter "[";
       pp_open_box formatter 2;
       let k', _ = coor_bande 0 j in
       let k = ref next.(k') in
       let i = ref (k' - !k) in
       let i' = ref (-1) in
       while !k >= 0 && !i < columns do
	 if !i - !i' > 1 then begin
	   if !i' >= 0 then begin
	     pp_print_string formatter ",";
	     pp_print_space formatter ();
	   end;
	   pp_print_string formatter "%";
	   pp_print_int formatter (!i - !i' - 1)
	 end;
	 if !i <> 0 then begin
	   pp_print_string formatter ",";
	   pp_print_space formatter ();
	 end;
	 let _, l = coor_bande !i j in
	 R.write formatter m.(!k).(l);
	 i' := !i;
	 let nk = next.(!k-1) in
	 i := !i + !k - nk;  
	 k := nk;
       done;
       if columns - !i' > 1 then begin
	 if !i' >= 0 then begin
	   pp_print_string formatter ",";
	   pp_print_space formatter ();
	 end;
	 pp_print_string formatter "%";
	 pp_print_int formatter (columns - !i' - 1)
       end;
       pp_close_box formatter ();
       pp_print_string formatter "]";
     done;
     pp_close_box formatter ();
     pp_print_cut formatter ();
     pp_print_string formatter "]"
       
   let print = write std_formatter
     
   let parse str = failwith "not implemented"
		     
   let read ch = parse (Stream.of_channel ch)
       
   let write_bin = output_value
		    
   let read_bin = input_value
		    
   let ( == ) m1 m2 =
     try
       for k = 0 to lines + columns - 2 do
	 let l1 = Array.length m1.(k) in
	 let l2 = Array.length m2.(k) in
	 if l1 = l2 then
	   for l = 0 to (l1-1)do
	     if not (R.(==) m1.(k).(l) m2.(k).(l)) then raise Exit
	   done
	 else if l1 = 0 then 
	   for l = 0 to (l2-1)do
	     if not (R.(==) R.zero m2.(k).(l)) then raise Exit
	   done
	 else
	   for l = 0 to (l1-1)do
	     if not (R.(==) R.zero m1.(k).(l)) then raise Exit
	   done
       done;
       true
     with Exit -> false
	 
    let normalize m = Array.map (Array.map R.normalize) m
			
end 
    
module Make_Square = 
  functor (R:Ring) ->
  functor (Domain:Vector with type scalar = R.elem) -> 
    Make(R)(Domain)(Domain)


module Make_Symmetric = 
  functor (R:Ring) ->
  functor (Domain:Vector with type scalar = R.elem) -> 
struct
  type scalar = R.elem
  type argument = Domain.elem
  type result = Domain.elem
		     
  let columns = Domain.dim
  let lines = Domain.dim
  module D = struct 
    let dim = columns * lines 
  end
  let dim = columns * lines

  type elem = R.elem array array
		
  let line_length k = k + 1

  let coor_bande i j = 
    if i >= j then 
      let k = j - i + columns - 1 in
      k, j
    else
      let k = i - j + columns - 1 in
      k, i
      
  let coor_matrix k l =
    (l + columns - 1 - k, l)

  let create init =
     let m = Array.create columns [||] in
     let g i j v =
       let k, l = coor_bande i j in
       if Array.length m.(k)<1 then
	   m.(k)<-Array.create (line_length k) R.zero;
       m.(k).(l)<- if i >= j then v else R.conjugate v; 
     in 
     init g;
     m
	 	          
   let null () =
     let m = Array.create columns [||] in
     for j= 0 to columns - 1 do
       m.(j)<- Array.create 0 R.zero 
     done;
     m
	          
   let copy m =      
     let r = Array.create columns  [||] in
     for j = 0 to columns - 1 do
       r.(j)<- Array.copy m.(j)
     done;
     r
	  
   let apply m v =
     let result = Domain.null () in
     for k=0 to columns - 1 do
       for l=0 to Array.length m.(k) - 1 do
	 let i, j = coor_matrix k l in
	 Domain.vput result j 
	   (R.(++) 
	      (Domain.vget result j) 
	      (R.( ** ) m.(k).(l) (Domain.vget v i)));
	 if (i <> j) then Domain.vput result i 
	   (R.(++) 
	      (Domain.vget result i) 
	      (R.( ** ) (R.conjugate m.(k).(l)) (Domain.vget v j)))
       done
     done;
     result

   let tapply v m = apply m v

   let mul_scal x m =
     for k = 0 to columns - 1 do
       for l = 0 to ((Array.length m.(k))-1)do
	 m.(k).(l)<- R.( ** ) x m.(k).(l)
       done
     done
	 
   let write formatter m =
     let next = Array.create columns 0 in
     let k' = ref (columns - 1)  in
     for k = columns - 1 downto 0 do
       if Array.length m.(k)>=1 then 
	 for k'' = !k' downto k do 
	   next.(k'') <- k;
	   k' := k - 1
	 done
     done; 
     for k'' = !k' downto 0 do 
       next.(k'') <- (-1)
     done;
     pp_print_string formatter "[";
     pp_open_hvbox formatter 2;
     pp_print_int formatter columns;
     pp_print_string formatter "x";
     pp_print_int formatter lines;
     pp_print_string formatter ":";
     for j=0 to lines-1 do
       if j <> 0 then pp_print_string formatter ",";
       pp_print_space formatter ();
       pp_print_string formatter "[";
       pp_open_box formatter 2;
       let k' = columns - 1 in
       let k = ref next.(k') in
       let i = ref (k' - !k + j) in
       let i' = ref (-1) in
       while !k >= 0 && !i < columns do
	 if !i - !i' > 1 then begin
	   if !i' >= 0 then begin
	     pp_print_string formatter ",";
	     pp_print_space formatter ();
	   end;
	   pp_print_string formatter "%";
	   pp_print_int formatter (!i - !i' - 1)
	 end;
	 if !i <> 0 then begin
	   pp_print_string formatter ",";
	   pp_print_space formatter ();
	 end;
	 let _, l = coor_bande !i j in
	 R.write formatter m.(!k).(l);
	 i' := !i;
	 let nk = next.(!k-1) in
	 i := !i + !k - nk;  
	 k := nk;
       done;
       if columns - !i' > 1 then begin
	 if !i' >= 0 then begin
	   pp_print_string formatter ",";
	   pp_print_space formatter ();
	 end;
	 pp_print_string formatter "%";
	 pp_print_int formatter (columns - !i' - 1)
       end;
       pp_close_box formatter ();
       pp_print_string formatter "]";
     done;
     pp_close_box formatter ();
     pp_print_cut formatter ();
     pp_print_string formatter "]"

	
    let print = write std_formatter
     
    let parse str = failwith "not implemented"
		      
    let read ch = parse (Stream.of_channel ch)
	
    let write_bin = output_value
		    
    let read_bin = input_value

    let ( == ) m1 m2 =
      try
	for k = 0 to columns - 1 do
	  let l1 = Array.length m1.(k) in
	  let l2 = Array.length m2.(k) in
	  if l1 = l2 then
	    for l = 0 to (l1-1)do
	      if not (R.(==) m1.(k).(l) m2.(k).(l)) then raise Exit
	    done
	  else if l1 = 0 then 
	    for l = 0 to (l2-1)do
	      if not (R.(==) R.zero m2.(k).(l)) then raise Exit
	    done
	  else
	    for l = 0 to (l1-1)do
	      if not (R.(==) R.zero m1.(k).(l)) then raise Exit
	    done
	done;
	true
      with Exit -> false

    let normalize m = Array.map (Array.map R.normalize) m
		  
end 
