open Input_util
open Format
open Algebra
open Vector
open Matrix



module Make(R:Ring) = 
  functor (Domain : Vector with type scalar = R.elem) ->
  functor (Image : Vector with type scalar = R.elem) ->
struct   		  
  type scalar = R.elem
  type argument = Domain.elem
  type result = Image.elem
		     
  let columns = Domain.dim
  let lines = Image.dim
  module D = struct 
    let dim = columns * lines 
  end
  let dim = columns * lines
		
  type elem = 
      { 
	atab : scalar array;
        icol : int array;
        ilig : int array;
      }

  let create init =
    let l = ref [] in (* create a list *)
    let g i j v = l:=(i,j,v)::!l in
    init g;
    let l = !l in
    let nnz = List.length l in
    let cmp (i,j,_) (i',j',_) = j < j' or (j = j' & i < i') in 
    let l = Sort.list cmp l in (*trie une liste *)
    let atab= Array.create nnz R.zero in
    let icol = Array.create nnz 0 in
    let ilig= Array.create (lines+1) 0 in
    let pos = ref 0 in
    let next_line = ref 0 in
    let fn (i,j,v) = 
      icol.(!pos) <- i;
      atab.(!pos) <- v;
      if j >= !next_line then 
	while !next_line <= j do
	  ilig.(!next_line) <- !pos;
	  incr next_line;
	done;
      
      incr pos
    in
    List.iter fn l;
    ilig.(!next_line)<- nnz;
    { atab = atab; ilig = ilig ; icol = icol}


  let null () =
    {
      ilig = Array.create (lines+1) 0;
      atab = Array.create 0 R.zero;
      icol = Array.create 0 0;
    }

  let copy m =      
    {
      ilig = Array.copy m.ilig;
      atab = Array.copy m.atab;
      icol = Array.copy m.icol;
    }                                                                                         
  let apply m v =
    let init fn =
      for j=0 to (lines-1) do
	let somme = ref R.zero in
	for k = m.ilig.(j) to m.ilig.(j+1)-1 do
	  somme  := R.(++) !somme (R.( ** ) m.atab.(k) 
				     (Domain.vget v m.icol.(k)))
	done;
	fn j ! somme
      done in
    Image.vcreate init

  let tapply v m =
    let somme = Domain.null () in
    for j =0 to lines-1 do
      for k = m.ilig.(j) to m.ilig.(j+1)-1 do
	let i = m.icol.(k) in
	Domain.vput somme i 
	      (R.(++) (Domain.vget somme i)  
		 (R.( ** ) (R.conjugate m.atab.(k)) (Image.vget v j)));
      done;
    done;
    somme
    
	  
  let mul_scal x m =
    for i = 0 to m.ilig.(lines)-1 do
      m.atab.(i)<-R.( ** ) x m.atab.(i)
    done
    
  let write formatter p =
    pp_print_string formatter "[";
    pp_open_hvbox formatter 2;
    pp_print_int formatter columns;
    pp_print_string formatter "x";
    pp_print_int formatter lines;
    pp_print_string formatter ":";
    for j=0 to lines-1 do
      if j <> 0 then pp_print_string formatter ",";
      pp_print_space formatter ();
      pp_print_string formatter "[";
      pp_open_box formatter 2;
      let i' = ref (-1) in
      for pos = p.ilig.(j) to p.ilig.(j+1)-1 do
	let i = p.icol.(pos) in
	if i - !i' > 1 then begin
	  if !i' >= 0 then begin
	    pp_print_string formatter ",";
	    pp_print_space formatter ();
	  end;
	  pp_print_string formatter "%";
	  pp_print_int formatter (i - !i' - 1)
	end;
	if i <> 0 then begin
	  pp_print_string formatter ",";
	  pp_print_space formatter ();
	end;
	R.write formatter p.atab.(pos);
	i' := i
      done;
      if columns - !i' > 1 then begin
	if !i' >= 0 then begin
	  pp_print_string formatter ",";
	  pp_print_space formatter ();
	end;
	pp_print_string formatter "%";
	pp_print_int formatter (columns - !i' - 1)
      end;
      pp_close_box formatter ();
      pp_print_string formatter "]";
    done;
    pp_close_box formatter ();
    pp_print_cut formatter ();
    pp_print_string formatter "]"

  let print = write std_formatter
		
  let parse str = failwith "not implemented"
		    
  let read ch = parse (Stream.of_channel ch)
      
  let write_bin = output_value
		    
  let read_bin = input_value
		   
  let normalize m =
    {
      ilig = Array.copy m.ilig;
      atab = Array.map R.normalize m.atab;
      icol = Array.copy m.icol;
    }                          
    
  let (==) m1 m2 =
    try
      for j = 0 to lines-1 do
	let k1 = ref(m1.ilig.(j)) and end1 = m1.ilig.(j+1)-1 in
	let k2 = ref(m2.ilig.(j)) and end2 = m2.ilig.(j+1)-1 in
	while !k1 <= end1 or !k2 <= end2 do
	  match m1.icol.(!k1) - m1.icol.(!k2) with
	    0 -> 
	      if not (R.(==) m1.atab.(!k1) m2.atab.(!k2)) then raise Exit;
	      incr k1; incr k2
	  | x when x < 0 ->
	      if not (R.(==) m1.atab.(!k1) R.zero) then raise Exit;
	      incr k1
	  | _ ->
	      if not (R.(==) R.zero m2.atab.(!k2)) then raise Exit;
	       incr k2
	done
      done;
       true
    with Exit -> false
end
    
    
module Make_Square = 
  functor (R:Ring) ->
  functor (Domain:Vector with type scalar = R.elem) -> 
    Make(R)(Domain)(Domain)


module Make_Symmetric = 
  functor (R:Ring) ->
  functor (Domain:Vector with type scalar = R.elem) -> 
struct
  type scalar = R.elem
  type argument = Domain.elem
  type result = Domain.elem
		     
  let columns = Domain.dim
  let lines = Domain.dim
  module D = struct 
    let dim = columns * lines 
  end
  let dim = columns * lines
		
  type elem = 
      { 
	atab : scalar array;
        icol : int array;
        ilig : int array;
      }

  let create init =
    let l = ref [] in (* create a list *)
    let g i j v =
      if j>=i then (* we put in the list only the inf of the matrix *)
	l:=(i,j,v)::!l 
      else 
	l:=(j,i,R.conjugate v)::!l 
    in
    init g;
    let l = !l in
    let cmp (i,j,_) (i',j',_) = j > j' or (j = j' & i > i') in 
    let l = Sort.list cmp l in (*trie une liste *)
    (* eliminer les doublons ! *)
    let rec fn acc l =
      match acc, l with
	_, [] -> acc
      | (i,j,_)::_, (i',j',_)::l when i = i' && j = j' -> fn acc l
      | _, c::l -> fn (c::acc) l
    in
    let l = fn [] l in
    let nnz = List.length l in
    let atab= Array.create nnz R.zero in
    let icol = Array.create nnz 0 in
    let ilig= Array.create (lines+1) 0 in
    let pos = ref 0 in
    let next_line = ref 0 in
    let fn (i,j,v) = 
      if j >= !next_line or icol.(!pos-1) < i then begin
	icol.(!pos) <- i;
	atab.(!pos) <- v;
	if j >= !next_line then 
	  while !next_line <= j do
	    ilig.(!next_line) <- !pos;
	    incr next_line;
	  done;
	incr pos
      end
    in
    List.iter fn l;
    ilig.(!next_line)<- nnz;
    { atab = atab; ilig = ilig ; icol = icol}


  let null () =
    {
      ilig = Array.create (lines+1) 0;
      atab = Array.create 0 R.zero;
      icol = Array.create 0 0;
    }

  let copy m =      
    {
      ilig = Array.copy m.ilig;
      atab = Array.copy m.atab;
      icol = Array.copy m.icol;
    }                                                                                         
  let apply m v =
    let r = Domain.null () in
    for j=0 to (lines-1) do
      let somme = ref R.zero in
      for k = m.ilig.(j) to m.ilig.(j+1)-1 do
	let i =  m.icol.(k) in
	let x = m.atab.(k) in
	somme  := R.(++) !somme (R.( ** ) x (Domain.vget v i));
	if i <> j then
	  Domain.vput r i (R.(++) 
			    (Domain.vget r i) 
			    (R.( ** ) (R.conjugate x) (Domain.vget v j)));
      done;
      Domain.vput r j (R.(++) (Domain.vget r j) !somme);
    done;
    r

  let tapply v m = apply m v
    
  let mul_scal x m =
    for i = 0 to m.ilig.(lines)-1 do
      m.atab.(i)<-R.( ** ) x m.atab.(i)
    done
	  
  let write formatter p =
    pp_print_string formatter "[";
    pp_open_hvbox formatter 2;
    pp_print_int formatter columns;
    pp_print_string formatter "x";
    pp_print_int formatter lines;
    pp_print_string formatter ":";
    for j=0 to lines-1 do
      if j <> 0 then pp_print_string formatter ",";
      pp_print_space formatter ();
      pp_print_string formatter "[";
      pp_open_box formatter 2;
      let i' = ref (-1) in
      for pos = p.ilig.(j) to p.ilig.(j+1)-1 do
	let i = p.icol.(pos) in
	if i - !i' > 1 then begin
	  if !i' >= 0 then begin
	    pp_print_string formatter ",";
	    pp_print_space formatter ();
	  end;
	  pp_print_string formatter "%";
	  pp_print_int formatter (i - !i' - 1)
	end;
	if i <> 0 then begin
	  pp_print_string formatter ",";
	  pp_print_space formatter ();
	end;
	R.write formatter p.atab.(pos);
	i' := i
      done;
      if columns - !i' > 1 then begin
	if !i' >= 0 then begin
	  pp_print_string formatter ",";
	  pp_print_space formatter ();
	end;
	pp_print_string formatter "%";
	pp_print_int formatter (columns - !i' - 1)
      end;
      pp_close_box formatter ();
      pp_print_string formatter "]";
    done;
    pp_close_box formatter ();
    pp_print_cut formatter ();
    pp_print_string formatter "]"
      
  let print = write std_formatter
		
  let parse str = failwith "not implemented"
		    
  let read ch = parse (Stream.of_channel ch)
      
  let write_bin = output_value
		    
  let read_bin = input_value
		   
  let normalize m =
    {
      ilig = Array.copy m.ilig;
      atab = Array.map R.normalize m.atab;
      icol = Array.copy m.icol;
    }                          
    
  let (==) m1 m2 =
    try
      for j = 0 to lines-1 do
	let k1 = ref(m1.ilig.(j)) and end1 = m1.ilig.(j+1)-1 in
	let k2 = ref(m2.ilig.(j)) and end2 = m2.ilig.(j+1)-1 in
	while !k1 <= end1 or !k2 <= end2 do
	  match m1.icol.(!k1) - m1.icol.(!k2) with
	    0 -> 
	      if not (R.(==) m1.atab.(!k1) m2.atab.(!k2)) then raise Exit;
	      incr k1; incr k2
	  | x when x < 0 ->
	      if not (R.(==) m1.atab.(!k1) R.zero) then raise Exit;
	      incr k1
	  | _ ->
	      if not (R.(==) R.zero m2.atab.(!k2)) then raise Exit;
	       incr k2
	done
      done;
      true
    with Exit -> false
   		  

end 







