(**************************************************
**              polynome.ml                     ***
**            Polynome module                   ***
**      POUSSARD PA    march 2000               ***
**************************************************)

open Input_util
open Algebra
open Format

module type Polynomial  =
sig
  include Vectorial_Space
    
  type stathm = int (* type of polynome stathm that is it's degree ! *)
		  
  type init = (int -> scalar -> unit ) -> unit		
      (*ring struct*)
  val one : elem
  val t_of_int : int -> elem
  val ( ** ) : elem -> elem -> elem
      
  val degree : elem -> stathm
  val stathm : elem -> stathm
  val less  : stathm -> stathm -> bool
      
  val ( // ) : elem -> elem -> elem (* div quotient *)
  val (mod)   : elem -> elem -> elem (* div remainder *)
  val div_mod : elem -> elem -> elem * elem (* fst : // ,snd : mod *) 
      
  val derive : elem -> elem (* derivate *)
  val int  : elem -> elem (* intergrate *)
      
  val vcreate: init -> int -> elem (* creation function. works like
				    Vector.vcreate *)
  val monome : scalar -> int ->elem (* elem = t*x^(int) *)
  val copy : elem -> elem (* copy in new struct *)
      
  val vget : elem -> int -> scalar
	  (* raise Invalid_argument if int < 0 *)
  val vput :  elem -> int -> scalar -> unit
      (* raise Invalid_argument if int < 0 or int > degree *)
end
  
module type Normed_Polynomial =
sig
  include Banach

  type stathm = int (* type of polynome stathm that is it's degree ! *)
		  
  type init = (int -> scalar -> unit ) -> unit		
      (*ring struct*)
  val one : elem
  val t_of_int : int -> elem
  val ( ** ) : elem -> elem -> elem
      
  val degree : elem -> stathm
  val stathm : elem -> stathm
  val less  : stathm -> stathm -> bool
      
  val ( // ) : elem -> elem -> elem (* div quotient *)
  val (mod)   : elem -> elem -> elem (* div remainder *)
  val div_mod : elem -> elem -> elem * elem (* fst : // ,snd : mod *) 

  val derive : elem -> elem (* derivate *)
  val int  : elem -> elem (* intergrate *)
      
  val vcreate: init -> int -> elem (* creation function. works like
				    Vector.vcreate *)
  val monome : scalar -> int ->elem (* elem = t*x^(int) *)
  val copy : elem -> elem (* copy in new struct *)
      
  val vget : elem -> int -> scalar
      (* raise Invalid_argument if int < 0 *)
  val vput :  elem -> int -> scalar -> unit
      (* raise Invalid_argument if int < 0 or int > degree *)
end
  
module Make = functor (R :Field) ->  
struct 	           
  type scalar  = R.elem (* type of polynome coef *)
  type stathm = int (* stathm type *)
		  
  type init = (int -> scalar -> unit ) -> unit		 
  type elem = scalar array 
		
  let zero = [| |]
  let one =  [| R.one |]
  let t_of_int i = [| R.t_of_int i |]
		     
     (**************  intermediaires functions ********************)
		     
  let null deg = (* polynome allocation *)
    let p = Array.create (deg + 1) R.zero in
    p
      
  let apply deg fn p1 p2 p3 = (* create polynome with some function and this degree *)
    for i = 0 to (deg) do
      fn p1 p2 p3 i 
    done 
    
  let copy = Array.copy
	       
  let degree p = (Array.length p - 1) 
		   
		   
     (**************  input output fonctions  ********************)
		   
  let vget p deg = 
    if (deg < 0) 
    then raise (Invalid_argument "Polynome.get")
    else
      if (deg > (degree p)) 
      then R.zero 
      else  p.(deg)
	
	
  let vput p deg value = 
    if  (deg < 0) || ((degree p) < deg )
    then raise (Invalid_argument "Polynome.vput")
    else p.(deg) <- value
      
      
  let normalize' p =
    let degree = degree p in
    if R.(==) p.(degree) R.zero then begin
      let last = ref (degree-1) in
      while !last >= 0 && R.(==) p.(!last) R.zero do
	decr last
      done;
      Array.sub p 0 (!last+1)
    end else 
      p
	
  let normalize p =
    normalize' (Array.map R.normalize p)
      
     (***************** fst methodes *************************)
      
  let arrayToPoly tab =
    (tab : elem)
    
  let vcreate init deg =
    let m = null deg in
    init (fun i x -> m.(i) <- x);
    m
      
  let monome c n = (* c*X^n *)
    let p = null n in
    p.(n) <- c; 
    p
      
  let (++) p1 p2 =
    let deg_p1 = degree p1 in
    let deg_p2 = degree p2 in
    let result, other = 
      if (deg_p1 >= deg_p2)
      then 
	copy p1, p2
      else 
	copy p2, p1 
    in
    for i = 0 to min deg_p1 deg_p2 do
      result.(i) <- R.(++) result.(i) other.(i)
    done;
    normalize' result
      
  let opp p =
    Array.map R.opp p
      
  let (--) p1 p2 =
    begin
      let deg_p1 = degree p1 in
      let deg_p2 = degree p2 in
      let result, other, op = 
	if (deg_p1 >= deg_p2) then 
	  copy p1, p2, R.(--)
	else 
	  opp p2, p1, R.(++) 
      in
      for i = 0 to min deg_p1 deg_p2 do
	result.(i) <- op result.(i) other.(i)
      done;
      normalize' result
    end

  let (@) scal p  =
    Array.map (fun x -> R.( ** ) scal x) p
      
  let (==) p1 p2 =
    try
      let deg_p1 = degree p1 in
      let deg_p2 = degree p2 in
      if (deg_p1 <> deg_p2) then raise Exit;
      for i = 0 to deg_p1 do       
	if not (R.(==) p1.(i) p2.(i)) then raise Exit   
      done; true
    with Exit -> false
	
  let ( ** ) p q =
    let deg_p = degree p in
    let deg_q = degree q in
    if (deg_p <> -1) && (deg_q <> -1) (* no null factor *)
    then
      begin
	let dim = deg_p + deg_q in
	let produit = null dim in
	for i = 0 to deg_p do
	  for j =0 to deg_q do
	    produit.(i+j) <- R.(++) (R.( ** ) p.(i) q.(j)) produit.(i+j)
	  done
	done;
	produit
      end
    else 
      [| |]
      
  let stathm p = degree p
		   
  let less = (<)
	       
	       
    (*************  integr methodes     *******************)
	       
  let derive p =
    let deg_p = degree p in
    if deg_p <= 0 then
      [| R.zero |]
    else
      begin
	let init fn = 
	  for i = 1 to deg_p do
	    fn (i-1) (R.( ** ) (R.t_of_int i) p.(i))
	    done in
	vcreate init (deg_p - 1)
      end
      
  let int p = (* with const = R.zero *)
    let deg_p = degree p in
    let init fn =
      for i = 1 to (deg_p+1) do
	fn (i) (R.( // ) p.(i-1) (R.t_of_int (i)))
      done in
    vcreate init (deg_p+1)
      
    (************* euclidians methodes *******************)
      
  let div_mod poly divi  =
    let deg_poly = degree poly in
    let deg_div  = degree divi  in
    let dim = deg_poly - deg_div in
    if deg_div < 0 then 
      raise (Invalid_argument "Polynomial.div_mod")
    else if dim < 0 then
      zero, poly
    else if deg_div = 0 or deg_poly < 0 then
      poly, zero
    else
      let rem = copy poly in
      let quot = null dim in
      let d = divi.(deg_div) in
      let rec div deg_rem deg_quot =
	if (deg_rem < deg_div)
	then 
	  quot, normalize' rem
	else
	  let coef = R.(//) rem.(deg_rem) d in
	  quot.(deg_quot) <- coef;
	  rem.(deg_rem) <- R.zero;
	  for i = deg_rem - deg_div to deg_rem - 1 do
	    rem.(i) <- R.(--) rem.(i) (R.( ** ) coef divi.(deg_div + i - deg_rem)) 
	  done;
	  div (deg_rem - 1) (deg_quot - 1)
      in 
      div (deg_poly) (dim)
	
  let (//) p q = fst (div_mod p q)
		   
  let (mod) p q = snd (div_mod p q)
		       
  (***********  channel output input ********************)
		    
  let read_bin ch = 
    input_value ch
      
  let write_bin ch p =
    output_value ch p 
	
  let write formatter p =
    let ind = Array.length p in
    pp_print_string formatter "[";
    pp_open_box formatter 2;
    pp_print_int formatter ind;
    pp_print_string formatter ":";
    for i= 0 to (ind-1) do
      if i <> 0 then pp_print_string formatter ",";
      pp_print_space formatter ();
      R.write formatter p.(i);
    done;
    pp_print_cut formatter ();
    pp_close_box formatter ();
    pp_print_string formatter "]"

  let print = write std_formatter
		
  let parse str = failwith "not implemented"
		    
  let read ch = parse (Stream.of_channel ch)
      
  let conjugate p = Array.map R.conjugate p 

end


module Make_Normed = functor (R : Normed_Field) ->  
   struct 	           
     module P = Make(R)		  

     type scalar  = P.scalar
     type stathm = P.stathm
     type init = P.init
     type elem = P.elem
     type norm = R.norm
     let zero = P.zero
     let one =  P.one
     let t_of_int = P.t_of_int		
     let copy = P.copy
     let degree = P.degree
     let vget = P.vget       
     let vput = P.vput
     let normalize = P.normalize	   
     let arrayToPoly = P.arrayToPoly
     let vcreate = P.vcreate	   
     let monome = P.monome	   
     let (++) = P.(++)
     let opp = P.opp
     let (--) = P.(--)
     let (@) = P.(@)
     let (==) = P.(==)
     let ( ** ) = P. ( ** )
     let stathm = P.stathm		   
     let less = P.less
     let derive = P.derive	      
     let int = P.int  
     let div_mod = P.div_mod	          
     let (//) = P.(//)		      
     let (mod) = P.(mod)    
     let write = P.write
     let print = P.print		  
     let parse = P.parse
     let read = P.read
     let read_bin = P.read_bin 
     let write_bin = P.write_bin
     let conjugate = P.conjugate

     let (||) p q =
       let dim = min (degree p) (degree q) in
       let temp = ref R.zero in
       for i = 0 to dim do 
         temp := R.(++) !temp (R.( ** ) (R.conjugate p.(i)) q.(i))
       done;
       !temp 

     let norm p =
       let temp = ref R.zero_norm in
       for i = 0 to degree p do 
         temp := R.add_norm !temp (R.norm p.(i))
       done;
       !temp 

     let abs x = R.abs x 

   end








