open Input_util
open Format
open Algebra
open Vector
open Matrix
     

module type Iterative_Solver =
  sig
    type elem
    type argument
    type result
    type norm

    val solver : elem -> result -> norm -> argument
  end

module GC = 
  functor (R : Normed_Field) ->
  functor (V : Normed_Vector with type scalar = R.elem and type norm = R.norm) ->
  functor (M : Light_Matrix with type scalar = R.elem 
		     and type argument = V.elem
		     and type result = V.elem) ->
struct
  open M
  open V
  open R

  type elem = M.elem
  type norm = R.norm
  type argument = V.elem
  type result = V.elem

  let solver a b e = 
    let n = ref 0 in
    let x = null () in
    let r = copy b in
    let rho = ref (r || r) in
    let p = copy r in
    try 
      while true do
        incr n;
	let q = apply a p in
        let alpha = !rho // (p || q) in
	mapq2 (fun x_i p_i -> x_i ++ (alpha ** p_i)) x p;
	mapq2 (fun r_i q_i -> r_i -- (alpha ** q_i)) r q;
        let rho' = !rho in
	rho := (r || r);
	if leq (abs !rho) e then raise Exit;
	let beta = !rho // rho' in
	mapq2 (fun p_i r_i -> (beta ** p_i) ++ r_i) p r;
      done; 
      failwith "impossible"
    with Exit ->(* print_int !n;*) print_newline (); x
end 
	
module Double_Matrix = 
  functor (R : Ring) ->
  functor (M : Light_Matrix with type scalar = R.elem) ->
  functor (Domain : Vector 
	   with type scalar = R.elem
	   and type elem = M.result * M.argument) ->

(struct
   type scalar = M.scalar
   type argument = Domain.elem
   type result = Domain.elem
   type elem = M.elem

   let columns = Domain.dim
   let lines = Domain.dim
 
   let create init = failwith "create not implemented"
   let copy = M.copy
   let apply f (v,v') =  (M.apply f v', M.tapply v f) 
   let tapply c f = apply f c
   let print = M.print 
   let (==) = M.(==)
   let normalize = M.normalize
   let print = M.print
   let write = M.write
   let parse = M.parse
   let read = M.read
   let read_bin = M.read_bin
   let write_bin = M.write_bin
 end : Light_Matrix with type scalar = R.elem
		    and type argument = Domain.elem
		    and type result = Domain.elem
		    and type elem = M.elem)

module Double_GC
  (R : Normed_Field)
  (V : Normed_Vector with type scalar = R.elem and type norm = R.norm)
  (M : Light_Matrix with type scalar = R.elem 
		     and type argument = V.elem
		     and type result = V.elem) =
struct
  module V' = Vector.Make_Normed_Cartesian_Product(R)(V)(V)
  module M' = Double_Matrix(R)(M)(V')
  module G = GC(R)(V')(M')

  type elem = M.elem
  type norm = R.norm
  type argument = V.elem
  type result = V.elem

  let solver a b e = 
    let b' = (b, b) in
    let (_, s) = G.solver a b' e in
    s
end 




