open Format
open Algebra

module R = Field_R

module D11 = 
  struct
    let dim = 11
  end

module D15 = 
  struct
    let dim = 15
  end

module V11 = Vector.Make(R)(D11)

module V15 = Vector.Make(R)(D15)

let init n fn = 
  for i = 0 to n-1 do
    let x = abs_float ((float) (n/2-i)) /. (float) n *. 2.0 in
    fn i (1.0 -. x)
  done

let v11 =
    V11.vcreate (init 11)

let v15 =
    V15.vcreate (init 15)

let _ = 
  open_box 0;
  V11.print v11;
  print_newline ()

let _ = 
  open_box 0;
  V15.print v15;
  print_newline ()

module MF = Full_matrix.Make(R)(V11)(V15)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1115 =
  MF.create (init 11 15)

let _ = 
  MF.print m1115;
  print_newline ()

let _ = 
  V15.print (MF.apply m1115 v11);
  print_newline ()

module MFSQ = Full_matrix.Make_Square(R)(V11)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1111 =
  MFSQ.create (init 11 11)

let _ = 
  MFSQ.print m1111;
  print_newline ()

let _ = 
  V11.print (MFSQ.apply m1111 v11);
  print_newline ()


module MFS = Full_matrix.Make_Symmetric(R)(V11)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1111 =
  MFS.create (init 11 11)

let _ = 
  MFS.print m1111;
  print_newline ()

let _ = 
  V11.print (MFS.apply m1111 v11);
  print_newline ()

module MM = Morse_matrix.Make(R)(V11)(V15)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1115 =
  MM.create (init 11 15)

let _ = 
  MM.print m1115;
  print_newline ()

let _ = 
  V15.print (MM.apply m1115 v11);
  print_newline ()

module MMSQ = Morse_matrix.Make_Square(R)(V11)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1111 =
  MMSQ.create (init 11 11)

let _ = 
  MMSQ.print m1111;
  print_newline ()

let _ = 
  V11.print (MMSQ.apply m1111 v11);
  print_newline ()


module MMS = Morse_matrix.Make_Symmetric(R)(V11)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1111 =
  MMS.create (init 11 11)

let _ = 
  MMS.print m1111;
  print_newline ()

let _ = 
  V11.print (MMS.apply m1111 v11);
  print_newline ()

module MB = Band_matrix.Make(R)(V11)(V15)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1115 =
  MB.create (init 11 15)

let _ = 
  MB.print m1115;
  print_newline ()

let _ = 
  V15.print (MB.apply m1115 v11);
  print_newline ()

module MBSQ = Band_matrix.Make_Square(R)(V11)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1111 =
  MBSQ.create (init 11 11)

let _ = 
  MBSQ.print m1111;
  print_newline ()

let _ = 
  V11.print (MBSQ.apply m1111 v11);
  print_newline ()


module MBS = Band_matrix.Make_Symmetric(R)(V11)

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

let m1111 =
  MBS.create (init 11 11)

let _ = 
  MBS.print m1111;
  print_newline ()

let _ = 
  V11.print (MBS.apply m1111 v11);
  print_newline ()




