open Format
open Algebra

module R = Field_R

module D11 = 
  struct
    let dim = 11
  end

module V11 = Vector.Make_Normed(R)(D11)

let init n fn = 
  for i = 0 to n-1 do
    let x = abs_float ((float) (n/2-i)) /. (float) n *. 2.0 in
    fn i (1.0 -. x)
  done

let v11 =
    V11.vcreate (init 11)

let _ = 
  V11.print v11;
  print_newline ()

let init c l fn =
  for i = 0 to c-1 do
    for j = max 0 (i-3) to min (l-1) (i+3) do
      fn i j ((float) (i+j))
    done
  done

module MFSQ = Full_matrix.Make_Square(R)(V11)

let m1111 =
  MFSQ.create (init 11 11)

let _ = 
  MFSQ.print m1111;
  print_newline ()

module SFSQ = Solver.GC(R)(V11)(MFSQ)

let vr = SFSQ.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MFSQ.apply m1111 vr));
  print_newline ()

module MFS = Full_matrix.Make_Symmetric(R)(V11)

let m1111 =
  MFS.create (init 11 11)

let _ = 
  MFS.print m1111;
  print_newline ()

module SFS = Solver.GC(R)(V11)(MFS)

let vr = SFS.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MFS.apply m1111 vr));
  print_newline ()

module MMSQ = Morse_matrix.Make_Square(R)(V11)

let m1111 =
  MMSQ.create (init 11 11)

let _ = 
  MMSQ.print m1111;
  print_newline ()

module SMSQ = Solver.GC(R)(V11)(MMSQ)

let vr = SMSQ.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MMSQ.apply m1111 vr));
  print_newline ()

module MMS = Morse_matrix.Make_Symmetric(R)(V11)

let m1111 =
  MMS.create (init 11 11)

let _ = 
  MMS.print m1111;
  print_newline ()

module SMS = Solver.GC(R)(V11)(MMS)

let vr = SMS.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MMS.apply m1111 vr));
  print_newline ()

module MBSQ = Band_matrix.Make_Square(R)(V11)

let m1111 =
  MBSQ.create (init 11 11)

let _ = 
  MBSQ.print m1111;
  print_newline ()

module SBSQ = Solver.GC(R)(V11)(MBSQ)

let vr = SBSQ.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MBSQ.apply m1111 vr));
  print_newline ()

module MBS = Band_matrix.Make_Symmetric(R)(V11)

let m1111 =
  MBS.create (init 11 11)

let _ = 
  MBS.print m1111;
  print_newline ()

module SBS = Solver.GC(R)(V11)(MBS)

let vr = SBS.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MBS.apply m1111 vr));
  print_newline ()

