open Format
open Algebra

module R = Field_C

module D11 = 
  struct
    let dim = 11
  end

module V11 = Vector.Make_Normed(R)(D11)

let init n fn = 
  for i = 0 to n-1 do
    let i' = (float) i in
    fn i (i', ((float) n)-.1.0-.i')
  done

let v11 =
    V11.vcreate (init 11)

let _ = 
  V11.print v11;
  print_newline ()

let init c l fn =
  for i = 0 to c-1 do
    fn i i (4.0, 0.0);
    if i > 0 then fn i (i-1) (1.0,1.0);
    if i > 0 then fn (i-1) i (1.0,1.0);
  done

module MFSQ = Full_matrix.Make_Square(R)(V11)

let m1111 =
  MFSQ.create (init 11 11)

let _ = 
  MFSQ.print m1111;
  print_newline ()

module SFSQ = Solver.Double_GC(R)(V11)(MFSQ)

let vr = SFSQ.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MFSQ.apply m1111 vr));
  print_newline ()

module MMSQ = Morse_matrix.Make_Square(R)(V11)

let m1111 =
  MMSQ.create (init 11 11)

let _ = 
  MMSQ.print m1111;
  print_newline ()

module SMSQ = Solver.Double_GC(R)(V11)(MMSQ)

let vr = SMSQ.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MMSQ.apply m1111 vr));
  print_newline ()

module MBSQ = Band_matrix.Make_Square(R)(V11)

let m1111 =
  MBSQ.create (init 11 11)

let _ = 
  MBSQ.print m1111;
  print_newline ()

module SBSQ = Solver.Double_GC(R)(V11)(MBSQ)

let vr = SBSQ.solver m1111 v11 10e-10

let _ = 
  V11.print vr;
  print_newline ()

let _ = 
  V11.print (V11.(--) v11 (MBSQ.apply m1111 vr));
  print_newline ()

