open Format
open Algebra
open Expression
open Vector
open Matrix
open Full_matrix

(* on travaille en dimension 3 *)
let dim = 3 
module Dim = struct let dim = dim end

(* Les vecteur de R^3 *)
module V = Vector.Make_Normed(Field_R)(Dim)

(* Les matrices 3x3 *)
module M = Make_Square(Field_R)(V)

(* Les expressions  valeur de R *)
module E = Expression(Field_R)

(* Les champs de vecteurs *)
module VF = Vector.Make_Normed(E)(Dim)

let grad f =
  let dfx = E.derive f "x" in
  let dfy = E.derive f "y" in
  let dfz = E.derive f "z" in
  VF.from_array [| dfx, dfy, dfz |]

let eval_grad v x = 
  let xs = ["x", V.vget x 0; "y", V.vget x 1; "z", V.vget x 2] in
  VF.from_array [|
      E.eval (VF.vget v 0) xs;
      E.eval (VF.vget v 1) xs;
      E.eval (VF.vget v 2) xs
  |]

let epsilon = 10e-10

let middle a b = V.map2 (fun x y -> (x +. y) /. 2.0) a b 

let dighotomie f a b =
  let fa = eval f a and fb = eval f b in
  if abs_float(fa) < epsilon then a else 
  if abs_float(fb) < epsilon then b else
  let fa, a, fb, b =     
    if fa < 0.0 && fb > 0.0 then fa, a, fb, b else
    if fa > 0.0 && fb < 0.0 then fb, b, fa, a else
    raise Not_found
  in
  let rec fn fa a fb b =
    let c = middle a b in
    let fc = eval f c in 
    if abs_float(fc) < epsilon then c else
    if fc < 0.0 then fn fc c fb b else fn fa a fc c 
  in
  fn fa a fb b

