(****************************************************************************)
(*                                vector.mli                                *)
(****************************************************************************)
 
open Algebra

(****************************************************************************)
(*                          Module  Type Vector                             *)
(****************************************************************************)

(* a module to store one interger *)
module type Dim = 
sig
  val dim : int
end
  
module type Vector =
sig
  include Vectorial_Space
    
    (* scalar product (should be only for Banach but useful here) *)
  val (||) : elem -> elem -> scalar
      (* direct acces to the dimension *)
  val dim :  int         
    (* copy *)   
  val copy : elem -> elem
      (* universal creation : to use write a code like
	 
	 let init f =
	 ...
	 f i x (* called if x is at position i for all non zero element *)
	 ...
	 in
	 let m = create init
      *) 
  val vcreate : ((int -> scalar -> unit) -> unit) -> elem    (* create ... *)
      (* direct access and modification : unsafe ! *)
  val from_array : scalar array -> elem
  val vget : elem -> int -> scalar
  val vput : elem -> int -> scalar -> unit
      (* null n : null vector of size n *)
  val null : unit -> elem             
      (* set a vector to zero *) 
  val init_null : elem -> unit  
      (* add v v1 : v := v + v1 *)
  val add : elem -> elem -> unit
      (* sub v v1 : v := v - v1 *)
  val sub : elem -> elem -> unit
      (* mul_scal x v : v := x*v *)
  val mul_scal : scalar -> elem -> unit 
      (* mapq f (x1,...,xn) : (x1,...,xn) := (f(x1),...,f(xn)) *)
  val mapq : (scalar -> scalar) -> elem -> unit
      (* mapq2 f (x1,...,xn) (y1,...,yn) : 
	 (x1,...,xn) := (f(x1,y1),...,f(xn,yn)) *)
  val mapq2 : (scalar -> scalar -> scalar) -> elem -> elem -> unit
      (* same function, but purely functional *)
  val map : (scalar -> scalar) -> elem -> elem
  val map2 : (scalar -> scalar -> scalar) -> elem -> elem -> elem 
end
  
module type Normed_Vector =
sig 
  include Banach
    
    (* direct acces to the dimension *)
  val dim :  int         
    (* copy *)   
  val copy : elem -> elem
      (* universal creation : to use write a code like
	 
	 let init f =
	 ...
	 f i x (* called if x is at position i for all non zero element *)
	 ...
	 in
	 let m = create init
      *) 
  val vcreate : ((int -> scalar -> unit) -> unit) -> elem    (* create ... *)
      (* direct access and modification : unsafe ! *)
  val from_array : scalar array -> elem
  val vget : elem -> int -> scalar
  val vput : elem -> int -> scalar -> unit
      (* null n : null vector of size n *)
  val null : unit -> elem             
      (* set a vector to zero *) 
  val init_null : elem -> unit  
      (* add v v1 : v := v + v1 *)
  val add : elem -> elem -> unit
      (* sub v v1 : v := v - v1 *)
  val sub : elem -> elem -> unit
      (* mul_scal x v : v := x*v *)
  val mul_scal : scalar -> elem -> unit 
      (* mapq f (x1,...,xn) : (x1,...,xn) := (f(x1),...,f(xn)) *)
  val mapq : (scalar -> scalar) -> elem -> unit
      (* mapq2 f (x1,...,xn) (y1,...,yn) : 
	 (x1,...,xn) := (f(x1,y1),...,f(xn,yn)) *)
  val mapq2 : (scalar -> scalar -> scalar) -> elem -> elem -> unit
      (* same function, but purely functional *)
  val map : (scalar -> scalar) -> elem -> elem
  val map2 : (scalar -> scalar -> scalar) -> elem -> elem -> elem 
      (* scalar product *)
end
  
(* build the vector structure from a Ring and a dimension *)
module Make(R : Ring)(D : Dim): Vector 
  with type scalar = R.elem 

(* build the normed vector structure from a Normed_Field and a dimension *)
module Make_Normed(R : Normed_Field)(D : Dim): Normed_Vector 
 with type scalar = R.elem 
 and type norm = R.norm

(* build vector structure as cartesian product of existing structure *)
module Make_Cartesian_Product
  (R : Ring)
  (V0 : Vector with type scalar = R.elem)
  (V1 : Vector with type scalar = R.elem) :
  Vector with type scalar = R.elem
	 and type elem = V0.elem * V1.elem

(* build normed vector structure as cartesian product of existing structure *)
module Make_Normed_Cartesian_Product
  (R : Normed_Field)
  (V0 : Normed_Vector with type scalar = R.elem and type norm = R.norm)
  (V1 : Normed_Vector with type scalar = R.elem and type norm = R.norm) :
  Normed_Vector 
  with type scalar = R.elem
  and type norm = R.norm
  and type elem = V0.elem * V1.elem








